
/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 9 (sqlite cache)
 */

#include "squidwall.h"
#include "timestamp.h"
#include "hextab.h"
#include "fmtX.h"
#include "md5.h"

#if WANT_SQLITE

#define BLOB_MINLEN 500
#define BLOB_MAXLEN 10 * (1024*1024)

/**
 * We have to reopen the database on every request, which is ugly :(
 * - is there a way, that we can call sql_quit() once @ the end ?
 * - is the closing of the database really needed ?
 * - a lot todo here!
 */
void cache_content(request_t *r)
{
	unsigned char digest[16];
	char *e, hash[33], ts[TIMESTAMP];
	MD5_CTX md5;
	stralloc sa, path, blob, type;
	int i;

	/* too small for learning */
	if (r->content_blob.len < BLOB_MINLEN) return;

	/* too big for learning: 10MB */
	if (r->content_blob.len > BLOB_MAXLEN) return;

	debug(9,5, "cache_content() anfang");

	stralloc_init(&sa);
	stralloc_init(&path);
	stralloc_init(&blob);
	stralloc_init(&type);

	MD5_Init(&md5);
	MD5_Update(&md5, r->url_host.s, r->url_host.len);
	MD5_Update(&md5, "/", 1);
	MD5_Update(&md5, r->url_path.s, r->url_path.len);
	MD5_Final(digest, &md5);

	e=hash;
	for (i=0; i<16; i++) {
		*e = hextab[digest[i]/16]; ++e;
		*e = hextab[digest[i]%16]; ++e;
	}
	*e=0;

	timestamp(ts);
	ts[TIMESTAMP]=0;

	if (!fmt_to_sa(fmt_blob, &path, r->url_path.s, r->url_path.len))
		die_nomem();
	if (!stralloc_0(&path)) die_nomem();

	if (!fmt_to_sa(fmt_blob, &blob, r->content_blob.s, r->content_blob.len))
		die_nomem();
	if (!stralloc_0(&blob)) die_nomem();

	if (!fmt_to_sa(fmt_blob, &type, r->content_type.s, r->content_type.len))
		die_nomem();
	if (!stralloc_0(&type)) die_nomem();

	sql_init();

	/**
	 * macht keinen sinn, da wir nur ein insert; haben
	 *
	 * if (!stralloc_copys(&sa, "BEGIN DEFERRED TRANSACTION")) die_nomem();
	 * sql_exec(&sa);
	 */

	if (!stralloc_copys(&sa, "REPLACE INTO tbl_cache (hash, ts, url_host,")) die_nomem();
	if (!stralloc_cats(&sa, " url_path, mimetype, content) VALUES ('")) die_nomem();
	if (!stralloc_cats(&sa, hash)) die_nomem();
	if (!stralloc_cats(&sa, "', '")) die_nomem();
	if (!stralloc_cats(&sa, ts)) die_nomem();
	if (!stralloc_cats(&sa, "', '")) die_nomem();
	if (!stralloc_cats(&sa, r->url_host.s)) die_nomem();
	if (!stralloc_cats(&sa, "', ")) die_nomem();
	if (!stralloc_cats(&sa, path.s)) die_nomem();
	if (!stralloc_cats(&sa, ", ")) die_nomem();
	if (!stralloc_cats(&sa, type.s)) die_nomem();
	if (!stralloc_cats(&sa, ", ")) die_nomem();
	if (!stralloc_cats(&sa, blob.s)) die_nomem();
	if (!stralloc_cats(&sa, ")")) die_nomem();
	sql_exec(&sa);

	debug(9,6, "cache_content(), hash[", hash, "]");
	debug(9,6, "cache_content(),   ts[", ts, "]");
	debug(9,6, "cache_content(), host[", r->url_host.s, "]");
	debug(9,6, "cache_content(), path[", r->url_path.s, "]");
	debug(9,6, "cache_content(), type[", r->content_type.s, "]");
	debug(9,6, "cache_content(), blob[", r->content_blob.s, "]");

	sql_quit();
	debug(9,6, "cache_content() ende");

	stralloc_free(&sa);
	stralloc_free(&blob);
	stralloc_free(&path);
	stralloc_free(&type);

	return;
}
#endif
