
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 11
 */

#include "squidwall.h"

/**
 * url banner checking
 */
int check_banner(request_t *r)
{
	struct squidwall_string *string;
	struct squidwall_regex *regex;
	struct named_regex *nr;
	struct list_head *l;
	unsigned int j, k;
	char *file, *str;

	if (r->bfilter_host)
	for (j=0; j < r->bfilter_host->len; j += str_len(r->bfilter_host->s+j)+1)
	list_for_each(l, &bfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->bfilter_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				return ACL_DENY;
			}
		}
	}

	if (r->bfilter_user)
	for (j=0; j < r->bfilter_user->len; j += str_len(r->bfilter_user->s+j)+1)
	list_for_each(l, &bfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->bfilter_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				return ACL_DENY;
			}
		}
	}

	if (r->brfilter_host)
	for (j=0; j < r->brfilter_host->len; j += str_len(r->brfilter_host->s+j)+1)
	list_for_each(l, &brfilter) {
		file = r->brfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				return ACL_DENY;
			}
		}
	}

	if (r->brfilter_user)
	for (j=0; j < r->brfilter_user->len; j += str_len(r->brfilter_user->s+j)+1)
	list_for_each(l, &brfilter) {
		file = r->brfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(regex_t)); k++) {
			nr = array_get(&regex->regex, sizeof(regex_t), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				return ACL_DENY;
			}
		}
	}

	if (r->bfilter_globals)
	for (j=0; j < r->bfilter_globals->len; j += str_len(r->bfilter_globals->s+j)+1)
	list_for_each(l, &bfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->bfilter_globals->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				return ACL_DENY;
			}
		}
	}

	if (r->brfilter_globals)
	for (j=0; j < r->brfilter_globals->len; j += str_len(r->brfilter_globals->s+j)+1)
	list_for_each(l, &brfilter) {
		file = r->brfilter_globals->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(regex_t)); k++) {
			nr = array_get(&regex->regex, sizeof(regex_t), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				return ACL_DENY;
			}
		}
	}

	return ACL_ALLOW;
}
