
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 11
 */

#include "squidwall.h"
#include "md5.h"
#include "sha1.h"
#include "hextab.h"

/**
 * url blacklist checking
 */
int check_blist(request_t *r)
{
	struct squidwall_string *string;
	struct squidwall_regex *regex;
	struct named_regex *nr;
	struct list_head *l;
	unsigned int j, k;
	char *file, *str, *e;
	char md5_url[33], sha1_url[41];
	unsigned char digest[20];
	MD5_CTX md5;
	SHA1_CTX sha1;

	/* setup the md5 hash */
	if (r->ufilter_md5_user || r->ufilter_md5_host || r->ufilter_md5_globals) {
		MD5_Init(&md5);
		MD5_Update(&md5, r->url_host.s, strlen(r->url_host.s));
		MD5_Final(digest, &md5);
		e=md5_url;
		for (j=0; j<16; j++) {
			*e = hextab[digest[j]/16]; ++e;
			*e = hextab[digest[j]%16]; ++e;
		}
		*e=0;
		debug(10, 5, "@check_blist() md5url(%s)", md5_url);
	}

	/* test the md5 lists (user acl's) */
	if (r->ufilter_md5_user)
	for (j=0; j < r->ufilter_md5_user->len; j += str_len(r->ufilter_md5_user->s+j)+1)
	list_for_each(l, &ufilter_md5) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_md5_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(md5_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the md5 lists (host acl's) */
	if (r->ufilter_md5_host)
	for (j=0; j < r->ufilter_md5_host->len; j += str_len(r->ufilter_md5_host->s+j)+1)
	list_for_each(l, &ufilter_md5) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_md5_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(md5_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	if (r->ufilter_md5_globals)
	for (j=0; j < r->ufilter_md5_globals->len; j += str_len(r->ufilter_md5_globals->s+j)+1)
	list_for_each(l, &ufilter_md5) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_md5_globals->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(md5_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* setup the sha1 hash */
	if (r->ufilter_sha1_user || r->ufilter_sha1_host || r->ufilter_sha1_globals) {
		SHA1_Init(&sha1);
		SHA1_Update(&sha1, r->url_host.s, strlen(r->url_host.s));
		SHA1_Final(digest, &sha1);
		e=sha1_url;
		for (j=0; j<20; j++) {
			*e = hextab[digest[j]/20]; ++e;
			*e = hextab[digest[j]%20]; ++e;
		}
		*e=0;
	}

	/* test the sha1 lists (user acl's) */
	if (r->ufilter_sha1_user)
	for (j=0; j < r->ufilter_sha1_user->len; j += str_len(r->ufilter_sha1_user->s+j)+1)
	list_for_each(l, &ufilter_sha1) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_sha1_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(sha1_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the sha1 lists (host acl's) */
	if (r->ufilter_sha1_host)
	for (j=0; j < r->ufilter_sha1_host->len; j += str_len(r->ufilter_sha1_host->s+j)+1)
	list_for_each(l, &ufilter_sha1) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_sha1_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(sha1_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	if (r->ufilter_sha1_globals)
	for (j=0; j < r->ufilter_sha1_globals->len; j += str_len(r->ufilter_sha1_globals->s+j)+1)
	list_for_each(l, &ufilter_sha1) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_sha1_globals->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(sha1_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url blacklists (user acl's) */
	if (r->ufilter_user)
	for (j=0; j < r->ufilter_user->len; j += str_len(r->ufilter_user->s+j)+1)
	list_for_each(l, &ufilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url blacklists (host acl's) */
	if (r->ufilter_host)
	for (j=0; j < r->ufilter_host->len; j += str_len(r->ufilter_host->s+j)+1)
	list_for_each(l, &ufilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	if (r->ufilter_globals)
	for (j=0; j < r->ufilter_globals->len; j += str_len(r->ufilter_globals->s+j)+1)
	list_for_each(l, &ufilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->ufilter_globals->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url regex blacklists (user acl's) */
	if (r->urfilter_user)
	for (j=0; j < r->urfilter_user->len; j += str_len(r->urfilter_user->s+j)+1)
	list_for_each(l, &urfilter) {
		file = r->urfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(regex_t)); k++) {
			nr = array_get(&regex->regex, sizeof(regex_t), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				if(!stralloc_copy(&r->filter, &regex->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cat(&r->filter, &nr->n)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url regex blacklists (host acl's) */
	if (r->urfilter_host)
	for (j=0; j < r->urfilter_host->len; j += str_len(r->urfilter_host->s+j)+1)
	list_for_each(l, &urfilter) {
		file = r->urfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				if(!stralloc_copy(&r->filter, &regex->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cat(&r->filter, &nr->n)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	if (r->urfilter_globals)
	for (j=0; j < r->urfilter_globals->len; j += str_len(r->urfilter_globals->s+j)+1)
	list_for_each(l, &urfilter) {
		file = r->urfilter_globals->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(regex_t)); k++) {
			nr = array_get(&regex->regex, sizeof(regex_t), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				if(!stralloc_copy(&r->filter, &regex->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cat(&r->filter, &nr->n)) die_nomem();
				r->filter.s[r->filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	return ACL_ALLOW;
}
