
/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 2
 */

#include "control.h"

/**
 * control_readint - reads some integer from the first line of a file
 *
 * @param filename  the full filepath of the file to be read from
 * @param value     the old/new value of the integer in that file
 *
 * @return  -1 on error
 *           0, the integer hasn't changed or can't be read
 *           1, the integer got some new value
 */
int control_readint(char *filename, int *value)
{
	stralloc line;
	int x=0;

	stralloc_init(&line);
	switch (control_readline(filename, &line, "")) {
		case 0:
			stralloc_free(&line);
			return 0;
		case -1:
			stralloc_free(&line);
			return -1;
	}

	if (!stralloc_0(&line)) die_nomem();

	if (scan_int(line.s, &x) == 0) {
		stralloc_free(&line);
		return 0;
	}

	stralloc_free(&line);
	if (*value != x) {
		/* some new value */
		*value=x; return 1;
	}

	return 0;
}
