
/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 2
 */

#include "control.h"

static void striptrailingwhitespace(stralloc *sa)
{
	while (sa->len > 0)
	switch(sa->s[sa->len - 1]) {
	case '\n':
	case '\t':
	case ' ':
		--sa->len;
		break;
	default:
		return;
	}
}

/**
 * control_readline - reads the full first line of a file
 *
 * @param filename  the full filepath of the file to be read from
 * @param line      this stralloc gets the content
 *
 * @return  -1 on error
 *           0, the stralloc line is the same as old
 *           1, line contains some new value now
 */
int control_readline(char *filename, stralloc *line, char *old)
{
	stralloc sa;
	buffer b;

	if (buffer_mmapread(&b, filename) == -1) {
		return -1;
	}

	stralloc_init(&sa);
	if (buffer_getnewline_sa(&b, &sa) == 1) {
		striptrailingwhitespace(&sa);
		if (!stralloc_copy(line, &sa)) die_nomem();
		buffer_close(&b);
		stralloc_free(&sa);
		if (stralloc_diffs(line, old) == 0) {
			return 0;
		} else {
			return 1;
		}
	}

	buffer_close(&b);
	stralloc_free(&sa);
	return -1;
}
