
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 8
 */

#include <stdarg.h>

/* libowfat */
#include "stralloc.h"

/* squidwall */
#include "read_controls.h"
#include "strallocX.h"
#include "log.h"

/* don't mix messages for the global logbuffer from diff. threads */
static pthread_mutex_t dbg_mutex = PTHREAD_MUTEX_INITIALIZER;

/**
 * debugging with some formatting ;)
 * - there only some minimum of formatting currently
 * - if we need more, adding some new features should be simple
 */
void debug_r(request_t *r, unsigned int section, int unsigned level, const char *fmt, ...)
{
	stralloc buf; /* the filled buffer */
	va_list ap;
	stralloc *sa;
	char *s;
	int d, len, pad=0;
	unsigned int u;

	/* should it be looged ? */
	if (debugging[section] < level) return;

	stralloc_init(&buf);

	// #include <stdio.h>
	// printf("@debug_r() r=%p (time:%d)\n", r, (int)time(0)); fflush(stdout);

	va_start(ap, fmt);
	while (*fmt) {
		if (*fmt != '%') {
			if (!stralloc_catb(&buf, fmt++, 1)) die_nomem();
			continue;
		}

		fmt++; /* skip '%' */
		len = scan_int(fmt, &d);
		if (len) {
			fmt+=len;
			pad = d;
		} else {
			pad = 0;
		}

		switch (*fmt) {
		case 's': /* string */
			s = va_arg(ap, char *);
			if (pad) {
				len = str_len(s);
				if (!stralloc_cat_pad(&buf, s, len, pad, pad-1)) die_nomem();
			} else {
				if (!stralloc_cats(&buf, s)) die_nomem();
			}
			break;
		case 'd': /* int */
			d = va_arg(ap, int);
			if (!stralloc_cat_int(&buf, d)) die_nomem();
			break;
		case 'u': /* unsigned int */
			u = va_arg(ap, unsigned int);
			if (!stralloc_cat_u32pw(&buf, u, pad?pad:0, ' ')) die_nomem();
			break;
		case 'x': /* int (hex) - add %8x ? */
			u = va_arg(ap, unsigned int);
			if (!stralloc_cat_x32pw(&buf, u, pad?pad:8, '0')) die_nomem();
			break;
		case 'S': /* stralloc */
			sa = va_arg(ap, stralloc *);
			if (!stralloc_cat(&buf, sa)) die_nomem();
			break;
		}
		fmt++;
	}
	va_end(ap);

	if (!stralloc_cats(&buf, "\n")) die_nomem();
	if (!stralloc_0(&buf)) die_nomem();

	pthread_mutex_lock(&dbg_mutex);
	log_headline(r, "");
	log_s(r, " ");
	log_s(r, buf.s);
	log_flush(r);
	pthread_mutex_unlock(&dbg_mutex);

	stralloc_free(&buf);

	return;
}
