#ifndef DEBUG_H
#define DEBUG_H

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* system */
#include <unistd.h>

/* squidwall */
#include "request.h"
#include "read_controls.h"

#define MAX_SECTION 50 /* 0..49 */

extern void debug_r(request_t *r, unsigned int section, unsigned int level, const char *fmt, ...);

/* developer may choose the last line :) */
#if 1
/* user */
#define debug(s,l,fmt,args...)  do { debug_r(r,s,l,fmt,##args); } while (0)
#else
/* developer */
#define debug(s,l,fmt,args...)  do { \
	debug_r(r,s,l,"%24s(@%4u)[%2u,%2u]: " fmt, \
	__func__, __LINE__, s, l, ##args); \
} while (0)
#endif

#define panic(s) debug(1,1,s)

/* these should never happen! */
#define die_nomem() _exit(111)
#define die_pthread() _exit(111)

#endif
