#ifndef SW_PRAMS_H
#define SW_PRAMS_H

/**
 * all these values can be overridden by environment variables!
 */

/**
 * default number of starting threads
 */
#define NUMTHREADS  10

/**
 * this logfile will show you the denied users/hosts/pages
 */
#define LOGFILE          "/var/log/squid/squidwall.log"

/**
 * ip:port of your squid cache
 *
 * warning: this cache should not re-redirect to the redirector :)
 */
#define PROXY            "127.0.0.1:3128"

/**
 * ip:port of clamd in TCP/IP mode
 */
#define CLAMAV_TCP       "127.0.0.1:3310"

/**
 * /some/path/to/clamd.sock of clamd in local socket mode
 */
#define CLAMAV_SOCKET    "/var/lib/clamav/clamd.sock"

/**
 * the default size in bytes, which will be passed through the content
 * filter and the clamav stream for antivirus checking
 */
#define MAXSIZE          60000

/**
 * how many url should be remebered in memory ... can speed up things :)
 */
#define MEMCACHE         100

/* phrase filter */
#define PFILTER_MAX      100 /* deadline for the pfilters, variable: 0..255 */
#define PFILTER_DEFAULT  25  /* for unacceptable values in configuration */

/* default timeout, for sqlite3 busy requests */
#define SQLITE_TIMEOUT   300

/* user / host acl's and redirects */
#define CACHE_DIR         "/var/cache/squidwall"
#define CONFIG_DIR        "/etc/squidwall"
#define HOST_DIR          CONFIG_DIR "/hosts"
#define USER_DIR          CONFIG_DIR "/users"
#define CONTROL_DIR       CONFIG_DIR "/control"
#define REDIR_DIR         CONFIG_DIR "/redirects"
#define GLOBAL_DIR        CONFIG_DIR "/globals"

/* content filters */
#define PFILTER_DIR       CONFIG_DIR "/filters/pfilter"
#define CFILTER_DIR       CONFIG_DIR "/filters/cfilter"
#define CRFILTER_DIR      CONFIG_DIR "/filters/crfilter"
#define BCFILTER_DIR      CONFIG_DIR "/filters/bcfilter"

/* url filters - blacklisting */
#define UFILTER_DIR       CONFIG_DIR "/filters/ufilter"
#define UFILTER_URL_DIR   CONFIG_DIR "/filters/ufilter_url"
#define UFILTER_DOM_DIR   CONFIG_DIR "/filters/ufilter_dom"
#define UFILTER_MD5_DIR   CONFIG_DIR "/filters/ufilter_md5"
#define UFILTER_SHA1_DIR  CONFIG_DIR "/filters/ufilter_sha1"
#define URFILTER_DIR      CONFIG_DIR "/filters/urfilter"

/* url filters - whitelisting */
#define WFILTER_DIR       CONFIG_DIR "/filters/wfilter"
#define WFILTER_URL_DIR   CONFIG_DIR "/filters/wfilter_url"
#define WFILTER_DOM_DIR   CONFIG_DIR "/filters/wfilter_dom"
#define WFILTER_MD5_DIR   CONFIG_DIR "/filters/wfilter_md5"
#define WFILTER_SHA1_DIR  CONFIG_DIR "/filters/wfilter_sha1"
#define WRFILTER_DIR      CONFIG_DIR "/filters/wrfilter"

/* url filters - banner */
#define BFILTER_DIR       CONFIG_DIR "/filters/bfilter"
#define BRFILTER_DIR      CONFIG_DIR "/filters/brfilter"

/* bayes learning filters */
#define BAYES_OSBF_DIR    CONFIG_DIR "/filters/bayes_osbf"

/* you can configure your deny page here */
#define REDIR_SERVER       "http://www.example.org"
#define REDIR_NETWORK      "x"
#define REDIR_BADURL       REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=url&"
#define REDIR_BADCONT      REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=cont&"
#define REDIR_BADHOST      REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=host&"
#define REDIR_BADUSER      REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=user&"
#define REDIR_VIRUSFOUND   REDIR_SERVER "/inet/deny.php?network=" REDIR_NETWORK "&reason=virus&"
#define REDIR_BANNERFOUND  REDIR_SERVER "/inet/img/t.png"

/* this url nerver redirected! */
#define REDIR_SAFESITE     REDIR_SERVER

/* this url is used for configuration via http://squidwall */
#define REDIR_WEBINTERFACE REDIR_SERVER "/inet"

/* default size for various buffers */
#define BUFSIZE 5000

#endif
