#ifndef SW_REQUEST_H
#define SW_REQUEST_H

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* squidwall */
#include "params.h"

/* libowfat */
#include "buffer.h"
#include "io.h"
#include "scan.h"
#include "stralloc.h"

typedef struct {

	/* current number in squid request */
	unsigned int tid;

	/* current worker, which handles this request */
	unsigned int wid;

	/* pointer to the infos from squid @ fd=0 */
	char *squid_ip;
	char *squid_url;
	char *squid_host;
	char *squid_user;
	char *squid_method;

	/* decoded infos */
	stralloc ip;
	stralloc url;
	stralloc host;
	stralloc user;
	stralloc method;

	/* decoded variables for the destination site */
	stralloc redirsite;
	stralloc filter;
	stralloc clamav;
	stralloc virus;

	/* splitted squid_url for use with md5/sha1 lists in check_blist() */
	stralloc url_host; /* -> www.example.org */
	stralloc url_path; /* -> /some/path/ */

	/* defined filters of current ip / host */
	stralloc *bfilter_host;
	stralloc *bcfilter_host;
	stralloc *brfilter_host;
	stralloc *cfilter_host;
	stralloc *crfilter_host;
	stralloc *pfilter_host;
	stralloc *ufilter_host;
	stralloc *ufilter_md5_host;
	stralloc *ufilter_sha1_host;
	stralloc *urfilter_host;
	stralloc *wfilter_host;
	stralloc *wrfilter_host;

	/* defined filters of current user */
	stralloc *bfilter_user;
	stralloc *bcfilter_user;
	stralloc *brfilter_user;
	stralloc *cfilter_user;
	stralloc *crfilter_user;
	stralloc *pfilter_user;
	stralloc *ufilter_user;
	stralloc *ufilter_md5_user;
	stralloc *ufilter_sha1_user;
	stralloc *urfilter_user;
	stralloc *wfilter_user;
	stralloc *wrfilter_user;

	/* defined global filters */
	stralloc *bfilter_globals;
	stralloc *bcfilter_globals;
	stralloc *brfilter_globals;
	stralloc *cfilter_globals;
	stralloc *crfilter_globals;
	stralloc *pfilter_globals;
	stralloc *ufilter_globals;
	stralloc *ufilter_md5_globals;
	stralloc *ufilter_sha1_globals;
	stralloc *urfilter_globals;
	stralloc *wfilter_globals;
	stralloc *wrfilter_globals;

	int cfilter_todo;
	int pfilter_todo;
	int pfilter_current;
	stralloc pfilter_matches;

	/* buffers for proxy connection */
	int64 fd_proxy;
	char proxy_oBUF[BUFSIZE]; buffer proxy_out;
	char proxy_iBUF[BUFSIZE]; buffer proxy_in;

	/* command and stream connection to clamd */
	int64 fd_clamd, fd_stream;
	char clamd_iBUF[BUFSIZE]; buffer clamd_in;
	char clamd_oBUF[BUFSIZE]; buffer clamd_out;
	char clams_oBUF[BUFSIZE]; buffer clams_out;

	/* buffers for logging */
	char log_BUF[BUFSIZE];
	buffer logbuf;

	/* for sqlite3 caching */
#if WANT_SQLITE
	stralloc content_type;
	stralloc content_blob;
#endif
} request_t;

/* handle_request.c */
extern void handle_request(request_t *r);

#endif /* SW_HANDLE_REQUEST_H */
