
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 1
 */

#include "squidwall.h"
#include "thread.h"

/* dummy request for logging from main.c/thread.c */
request_t R, *r;

stralloc request;
pthread_mutex_t request_mutex = PTHREAD_MUTEX_INITIALIZER;

static pthread_mutex_t request_mutex2 = PTHREAD_MUTEX_INITIALIZER;

/* int main(int argc, char **argv) */
int main(void)
{
	stralloc sa;
	int rv;

	/* we ignore sigpipe! */
	sig_pipeignore();

	/* we use non blocking IO */
	if (!io_fd(0)) return 111;
	if (!io_fd(1)) return 111;

	stralloc_init(&sa);
	stralloc_init(&request);

	/* just for buffering logging/debugging stuff */
	r = &R; log_init(r,1);
	read_acls(r);

#if WANT_SQLITE
	/* creates our tables, if needed and connects to the database */
	cache_init();
#endif

	thread_init();

	for (;;) {
		/* remove old stuff */
		pthread_mutex_lock(&request_mutex);

		pthread_mutex_lock(&request_mutex2);
		if (!stralloc_copys(&sa, "")) die_nomem();

		/* recheck the filter definitions and acl's */
		read_acls(r);

		/* wait for some line from squid */
		for (;;) {
			io_wantread(0); io_wait(); io_dontwantread(0);
			rv = buffer_getline_sa(buffer_0, &sa);
			if (rv == 0) _exit(0); /* exiting */
			if (rv == 1) break;
		}
		sa.s[sa.len-1]=0; /* remove the newline */
		debug(1,5, "looking the request variable ...");
		if (!stralloc_copy(&request, &sa)) die_nomem();
		for (;;) {
			rv = thread_operation(handle_request);
			if (rv == 0) break;    /* worker is started */
			if (rv == 1) continue; /* a new worker was started */
			die_pthread();
		}
		pthread_mutex_unlock(&request_mutex2);
	}

	/* never reached */
	return 0;
}
